﻿
; --------------------------------------------------------------------------------------------------------------
; IO.pbi : Implementation: File
; Author: cxalex @ purebasic.fr/german
; Version: 0.1
; --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; Changelog:
; # 0.1 :
;   - Initial Release
;} --------------------------------------------------------------------------------------------------------------

; EnableExplicit

; Help - Enumeration
Enumeration
  #PB_File_Open
  #pb_File_Read
  #pb_File_Create
EndEnumeration

; New File
Procedure IO_File(FileName$, Type = #PB_File_Open, Flags = #Null)
  Protected hFile
  ; Creates/Reads/OpensFile
  Select Type
    Case #PB_File_Open   : hFile = OpenFile(#PB_Any, FileName$, Flags)
    Case #PB_File_Read   : hFile = ReadFile(#PB_Any, FileName$, Flags)
    Case #PB_File_Create : hFile = CreateFile(#PB_Any, FileName$, Flags)
  EndSelect
  ; If File is Ready: Create IO - Object
  If hFile 
    Protected *IO.__IO = AllocateMemory(SizeOf(__IO))
    *IO\handle = hFile
    *IO\vTable = ?__vTable_IO_File
    *IO\optionalFunctionality = (#IO_GetLenght|#IO_GetPosition|#IO_SetPosition|#IO_ImportFromIO|#IO_ExportToIO)
    *IO\name$ = "FILE"
  EndIf
  ProcedureReturn *IO
EndProcedure

; File, Writter Procedure
Procedure __IO_File_Write(*IO.__IO, *Mem, MemSize.i)
  ProcedureReturn WriteData(*IO\handle, *Mem, MemSize)
EndProcedure

; File, Reader Procedure
Procedure __IO_File_Read(*IO.__IO, *Mem, MemSize.i)
  ProcedureReturn ReadData(*IO\handle, *Mem, MemSize)
EndProcedure

; File, Check Available Date
Procedure __IO_File_IsAvailable(*IO.__IO)
  ProcedureReturn (Lof(*IO\handle) - Loc(*IO\handle))
EndProcedure

; File, Check Total Legth
Procedure __IO_File_GetLength(*IO.__IO)
  ProcedureReturn Lof(*IO\handle)
EndProcedure

; File, Get Postion
Procedure __IO_File_GetPosition(*IO.__IO)
  ProcedureReturn Loc(*IO\handle)
EndProcedure

; File, Set Postion
Procedure __IO_File_SetPosition(*IO.__IO, Position)
  ProcedureReturn FileSeek(*IO\handle, Position)
EndProcedure

; File, Import From IO
Procedure __IO_File_ImportFromIO(*IO.__IO, *IO2.IO)
  ; IO supports Export?
  If *IO2\IOOptionalFunctions() & #IO_ExportToIO
    ; Delete all Data in File:
    FileSeek(*IO\handle, 0)
    TruncateFile(*IO\handle)
    ; Exports all Data from IO2 to IO
    *IO2\ExportToIO(*IO)
    ProcedureReturn #True
  EndIf
EndProcedure

; File, Export To Io
Procedure __IO_File_ExportToIO(*IO.__IO, *IO2.IO)
  Protected Seek = Loc(*IO\handle) ; Safe current Position
  Protected Size = Lof(*IO\handle) ; Size of File
  ; Reads all data from file:
  FileSeek(*IO\handle, 0)
  Protected *Mem = AllocateMemory(Size, #PB_Memory_NoClear)
  ReadData(*IO\handle, *Mem, Size)
  ; Sends Data to IO:
  *IO2\Write(*Mem, Size)
  ; Restore Position:
  FileSeek(*IO\handle, Seek)
  ; Frees Memory
  FreeMemory(*Mem)
EndProcedure

; File, Truncate
Procedure __IO_File_Truncate(*IO.__IO)
  ProcedureReturn TruncateFile(*IO\handle)
EndProcedure

; File, Close Procedure
Procedure __IO_File_Close(*IO.__IO)
  CloseFile(*IO\handle)
  ClearStructure(*IO, __IO)
  FreeMemory(*IO)
EndProcedure

; VTable:
DataSection
  __vTable_IO_File:
  Data.i @__IO_File_Write()
  Data.i @__IO_File_Read()
  Data.i @__IO_File_IsAvailable()
  Data.i @__IO_File_Truncate()
  Data.i @__IO_File_Close()
  Data.i @__IO_All_Handle()
  Data.i @__IO_All_Name()
  Data.i @__IO_All_GetOptionalFunctions()
  Data.i @__IO_File_GetLength()
  Data.i @__IO_File_GetPosition()
  Data.i @__IO_File_SetPosition()
  Data.i @__IO_File_ImportFromIO()
  Data.i @__IO_File_ExportToIO()
EndDataSection

; IDE Options = PureBasic 5.11 (Windows - x86)
; CursorPosition = 114
; FirstLine = 92
; Folding = ---
; EnableXP
; CPU = 1