﻿
; --------------------------------------------------------------------------------------------------------------
; IO.pbi : Implementation: SerialPort
; Author: cxalex @ purebasic.fr/german
; Version: 0.3
; --------------------------------------------------------------------------------------------------------------

;{ --------------------------------------------------------------------------------------------------------------
; Changelog:
; # 0.3 :
;   - Initial Release
;} --------------------------------------------------------------------------------------------------------------

; EnableExplicit

Structure __SerPort
  Handle.i
  *Buffer
  BufferSize.i
  BufferPos.i
  ReceiveSize.l
EndStructure

; New Serial Port
Procedure IO_SerialPort(SerialPortName$, Bauds, Parity, _Data.i, Stop.f, HandshakeMode, InputBufferSize, OutputBufferSize)
  Protected sPort = OpenSerialPort(#PB_Any, SerialPortName$, Bauds, Parity, _Data, Stop, HandshakeMode, InputBufferSize, OutputBufferSize)
  If sPort
    Protected *IO.__IO = AllocateMemory(SizeOf(__IO))
    Protected *SPort.__SerPort = AllocateMemory(SizeOf(__SerPort))
    *SPort\Handle = sPort
    *SPort\ReceiveSize = InputBufferSize
    *IO\handle = *SPort
    *IO\name$ = "SERIALPORT"
    *IO\optionalFunctionality = (#IO_ImportFromIO|#IO_ExportToIO)
    *IO\vTable = ?__vTable_IO_SerialPort
  EndIf
  ProcedureReturn *IO
EndProcedure

; Serial Port, Write
Procedure __IO_SerialPort_Write(*IO.__IO, *Mem, MemSize)
  Protected *SPort.__SerPort = *IO\handle
  ProcedureReturn WriteSerialPortData(*SPort\Handle, *Mem, MemSize)
EndProcedure

; Serial Port, Read
Procedure __IO_SerialPort_Read(*IO.__IO, *Mem, MemSize)
  Protected *SPort.__SerPort = *IO\handle
  If *SPort\BufferPos ; Buffer available
    If *SPort\BufferPos < MemSize
      MemSize = *SPort\BufferPos
    EndIf
    Protected NewMemSize = *SPort\BufferPos - MemSize
    ; Copy from internal Buffer to external
    CopyMemory(*SPort\Buffer, *Mem, MemSize)
    ; Is there something left in the Buffer?
    If Not NewMemSize
      FreeMemory(*SPort\Buffer)
      *SPort\Buffer = #Null
      *SPort\BufferPos = #Null
      *SPort\BufferSize = #Null
    Else ; Shrink Buffer
      MoveMemory(*SPort\Buffer + MemSize, *SPort\Buffer, NewMemSize)
      *SPort\BufferPos = NewMemSize
      *SPort\BufferSize = NewMemSize
      *SPort\Buffer = ReAllocateMemory(*SPort\Buffer, NewMemSize)
    EndIf
  EndIf
  ; Return Received Size
  ProcedureReturn MemSize
EndProcedure

; Serial Port, isAvailable
Procedure __IO_SerialPort_IsAvailable(*IO.__IO)
  Protected *SPort.__SerPort = *IO\handle
  
  While AvailableSerialPortInput(*SPort\Handle)
    ; Increase if necessary
    If *SPort\BufferSize < *SPort\BufferPos + *SPort\ReceiveSize
      *SPort\Buffer = ReAllocateMemory(*SPort\Buffer, *SPort\BufferPos + *SPort\ReceiveSize)
      *SPort\BufferSize = *SPort\BufferPos + *SPort\ReceiveSize
    EndIf
    ; Receive Data
    *SPort\BufferPos + ReadSerialPortData(*SPort\Handle, *SPort\Buffer + *SPort\BufferPos, *SPort\ReceiveSize)
  Wend 
  ProcedureReturn *SPort\BufferPos
EndProcedure

; SerialPort, Import
Procedure __IO_SerialPort_ImportFromIO(*IO.__IO, *IO2.IO)
  If *IO2\IOOptionalFunctions() & #IO_ExportToIO
    *IO2\ExportToIO(*IO)
    ProcedureReturn #True
  EndIf
EndProcedure

; Network, Export
Procedure __IO_SerialPort_ExportToIO(*IO.__IO, *IO2.IO)
  __IO_Network_IsAvailable(*IO)
  Protected *SPort.__SerPort = *IO\handle
  If *SPort\BufferPos ; Buffer Vorhanden
    *IO2\Write(*SPort\Buffer, *SPort\BufferPos)
  EndIf
EndProcedure

; Flush Buffer
Procedure __IO_SerialPort_Truncate(*IO.__IO)
  Protected *SPort.__SerPort = *IO\handle
  If *SPort\Buffer
    FreeMemory(*SPort\Buffer)
    *SPort\Buffer = #Null
    *SPort\BufferPos = #Null
    *SPort\BufferSize = #Null
  EndIf
EndProcedure

; Serial Port, Close
Procedure __IO_SerialPort_Close(*IO.__IO)
  Protected *SPort.__SerPort = *IO\handle
  __IO_SerialPort_Truncate(*IO)
  CloseSerialPort(*SPort\Handle)
  ClearStructure(*IO, __IO)
  FreeMemory(*IO)
EndProcedure

; SerialPort, Handle
Procedure __IO_SerialPort_Handle(*IO.__IO)
  Protected *SPort.__SerPort = *IO\handle
  ProcedureReturn *Sport\Handle
EndProcedure

; VTable, SerialPort:
DataSection
  __vTable_IO_SerialPort:
  Data.i @__IO_SerialPort_Write()
  Data.i @__IO_SerialPort_Read()
  Data.i @__IO_SerialPort_IsAvailable()
  Data.i @__IO_SerialPort_Truncate()
  Data.i @__IO_SerialPort_Close()
  Data.i @__IO_SerialPort_Handle()
  Data.i @__IO_All_Name()
  Data.i @__IO_All_GetOptionalFunctions()
  Data.i 0 ; GetLenght
  Data.i 0 ; GetPosition
  Data.i 0 ; SetPosition
  Data.i @__IO_SerialPort_ImportFromIO()
  Data.i @__IO_SerialPort_ExportToIO()
EndDataSection
; IDE Options = PureBasic 5.11 (Windows - x86)
; CursorPosition = 140
; FirstLine = 112
; Folding = --
; EnableXP
; CPU = 1